#include <sstream>
#include <iostream>
#include <fstream>
#include "ParametricShape.h"

ParametricShape::ParametricShape()
{
}

ParametricShape::ParametricShape(std::string filename)
{
	readFromFile(filename);
}

ParametricShape::ParametricShape(int nParameters, Eigen::VectorXd defaultParameters, std::vector<Eigen::Vector3i> triangles, 
	Eigen::MatrixXd verticesMatrix, Eigen::VectorXd verticesVector,	Eigen::MatrixXd constraintsMatrix, Eigen::VectorXd constraintsVector)
{
	defaultParameters_ = defaultParameters;
	nParameters_ = nParameters;
	triangles_ = triangles;
	verticesMatrix_ = verticesMatrix;
	verticesVector_ = verticesVector;
	constraintsMatrix_ = constraintsMatrix;
	constraintsVector_ = constraintsVector;

}

bool ParametricShape::evalShape(const Eigen::VectorXd &params, std::vector<Eigen::Vector3d> *vertices, std::vector<Eigen::Vector3i> *triangles)
{
	if ((constraintsMatrix_ ).rows() > 0)
	{

		if ((constraintsMatrix_ * params - constraintsVector_).maxCoeff() > EPSILON_CONST)
		{
			std::cout << "parameters are not valid " << std::endl;
			return false;
		}
	}

	Eigen::VectorXd vertexPositions = verticesMatrix_* params + verticesVector_; 
	int nVertices =  vertexPositions.size() / 3;
	vertices->resize(nVertices);
	
	for (int i = 0; i < nVertices; i++)
	{
		(*vertices)[i] = Eigen::Vector3d(vertexPositions[3*i], vertexPositions[3 * i + 1], vertexPositions[3 * i + 2]); 
	}
	*triangles = triangles_;

	return true;
}

bool ParametricShape::outputToFile(std::string filename)
{

	int nVertices = verticesVector_.size();
	int nTriangles = triangles_.size();
	int nConstraints = constraintsVector_.size();


	std::ofstream outFile(filename, std::ios::out | std::ios::binary);
	if (!outFile)
	{
		std::cout << "could not open file : " << filename << std::endl;
		return false;
	}

	outFile.write((char*) (&nParameters_), sizeof(int));
	outFile.write((char*) (&nVertices), sizeof(int));
	outFile.write((char*) (&nTriangles), sizeof(int));
	outFile.write((char*) (&nConstraints), sizeof(int));

	outFile.write((char *) defaultParameters_.data(),  nParameters_ * sizeof(double));
	outFile.write((char *) verticesMatrix_.data(), nVertices * nParameters_ * sizeof(double));
    outFile.write((char *) verticesVector_.data(), nVertices * sizeof(double));

	for (int i = 0; i < nTriangles; i++)
	{
		outFile.write((char *) triangles_[i].data(), 3 * sizeof(int));
	}

    outFile.write((char *) constraintsMatrix_.data(), nConstraints * nParameters_ * sizeof(double));
    outFile.write((char *) constraintsVector_.data(), nConstraints * sizeof(double));

    outFile.close();
}
	
bool ParametricShape::readFromFile(std::string filename)
{

	std::ifstream inFile(filename, std::ios::in | std::ios::binary);
	if (!inFile)
	{
		std::cout << "could not open file : " << filename << std::endl;
		return false;
	}

	int nVertices, nTriangles, nConstraints;

	inFile.read((char*) (&nParameters_), sizeof(int));
	inFile.read((char*) (&nVertices), sizeof(int));
	inFile.read((char*) (&nTriangles), sizeof(int));
	inFile.read((char*) (&nConstraints), sizeof(int));

	defaultParameters_.resize(nParameters_);
	inFile.read((char *) defaultParameters_.data(),  nParameters_ * sizeof(double));
	verticesMatrix_.resize(nVertices, nParameters_);
    inFile.read((char *) verticesMatrix_.data(), nVertices * nParameters_ * sizeof(double));
	verticesVector_.resize(nVertices);
    inFile.read((char *) verticesVector_.data(), nVertices * sizeof(double));

	triangles_.resize(nTriangles, Eigen::Vector3i::Zero()); 
	for (int i = 0; i < nTriangles; i++)
	{
		inFile.read((char *) triangles_[i].data(), 3 * sizeof(int));
	}


	constraintsMatrix_.resize(nConstraints, nParameters_);
	constraintsVector_.resize(nConstraints);
	inFile.read((char *) constraintsMatrix_.data(), nConstraints * nParameters_ * sizeof(double));
	inFile.read((char *) constraintsVector_.data(), nConstraints * sizeof(double));
	
    inFile.close();
	return true;
}











