#ifndef _PARAMETRIC_SHAPE_
#define _PARAMETRIC_SHAPE_

#include <vector>
#include <string>
#include <Eigen/Dense>

class ParametricShape{

public:
	ParametricShape(); 
	ParametricShape(std::string filename); 
	ParametricShape(int nParameters, Eigen::VectorXd defaultParameters, std::vector<Eigen::Vector3i> triangles, Eigen::MatrixXd verticesMatrix,
		Eigen::VectorXd verticesVector,	Eigen::MatrixXd constraintsMatrix, Eigen::VectorXd constraintsVector);
	bool evalShape(const Eigen::VectorXd &params, std::vector<Eigen::Vector3d> *vertices, std::vector<Eigen::Vector3i> *triangles);
	bool outputToFile(std::string filename);
	bool readFromFile(std::string filename);
	Eigen::VectorXd getDefaultParams() {return defaultParameters_;}
private:
	int nParameters_;
	Eigen::VectorXd defaultParameters_;
	std::vector<Eigen::Vector3i> triangles_;
	Eigen::MatrixXd verticesMatrix_;
	Eigen::VectorXd verticesVector_;
	Eigen::MatrixXd constraintsMatrix_;
	Eigen::VectorXd constraintsVector_;

};

const double EPSILON_CONST = 1e-7;

#endif